@@ -23,6 +23,7 @@ DEALINGS IN THE SOFTWARE.
 """
 
 import aiohttp
+from colorama import Fore, Style
 from typing import (
     Any, 
     Optional, 
@@ -32,13 +33,11 @@ from typing import (
 )
 from .anime import Anime, StreamLink
 from .character import Character
+from .core import Review, BASE
 from .manga import Manga
 from .error import *
 
 
-BASE: str = "https://kitsu.io/api/edge/"
-
-
 class Client:
     """Represents a client connection to get data from Kitsu.io API
 
@@ -49,10 +48,11 @@ class Client:
 
     """
 
-    def __init__(self, session: Optional[aiohttp.ClientSession] = None):
+    def __init__(self, session: Optional[aiohttp.ClientSession] = None) -> None:
         self._entries = {
             "anime": Anime,
             "manga": Manga,
+            "characters": Character
         }
         self._session: aiohttp.ClientSession = session or aiohttp.ClientSession()
         self._headers = {
@@ -66,35 +66,48 @@ class Client:
             if response.status == 200:
                 return response_data
             if response.status == 404:
-                return
+                return None
+            if response.status == 401:
+                raise NotAuthenticated
+            if response.status == 400:
+                raise BadApiRequest(response_data["errors"][0])
 
     async def search(
-        self, type: Literal["anime", "manga"], query: str, limit: int = 1
-    ) -> Optional[Union[Anime, List[Anime], Manga, List[Manga]]]:
+        self, 
+        type: Literal["anime", "manga", "characters"], 
+        query: str, 
+        limit: int = 1
+    ) -> Optional[Union[
+            Anime, List[Anime], 
+            Manga, List[Manga],
+            Character, List[Character]
+        ]]:
         """|coro|
 
         Search trough Kitsu API with the providen query and fetch the found data
 
         Parameters
         -----------
-        type: :class:`str`
-            The type of media to search
+        type: Literal["anime", "manga", "characters"]
+            The type of entry to search
         query: :class:`str`
             Represents the search query
         limit: :class:`int`
             Limit the search to a specific number of results
 
         """
-        entry = self._entries.get(type)
+        type_lower = type.lower()
+        entry = self._entries.get(type_lower)
+        filter = "name" if type_lower == "characters" else "text"
         fetched_data = await self._get_data(
-            f"{BASE}/{type}?filter%5Btext%5D={query}&page%5Blimit%5D={limit}"
+            f"{BASE}/{type_lower}?filter%5B{filter}%5D={query}&page%5Blimit%5D={limit}"
         )
         if not fetched_data["data"]:
             return None
         if len(fetched_data["data"]) == 1:
-            return entry(type, fetched_data["data"][0])
+            return entry(fetched_data["data"][0], self._session)
         return [
-            entry(type=type, attributes=attributes)
+            entry(attributes=attributes, session=self._session)
             for attributes in fetched_data["data"]
         ]
 
@@ -103,7 +116,7 @@ class Client:
     ) -> Optional[Union[Anime, List[Anime]]]:
         """|coro|
 
-        Shortcut function to :coro:`search` with the `type` parameter populated by the "anime" keyword
+        Shortcut function to :meth:`search` with the `type` parameter populated by the "anime" keyword
 
         Parameters
         -----------
@@ -119,7 +132,7 @@ class Client:
     ) -> Optional[Union[Manga, List[Manga]]]:
         """|coro|
 
-        Shortcut function to :coro:`search` with the `type` parameter populated by the "manga" keyword
+        Shortcut function to :meth:`search` with the `type` parameter populated by the "manga" keyword
 
         Parameters
         -----------
@@ -130,10 +143,30 @@ class Client:
         """
         return await self.search("manga", query=query, limit=limit)
 
-    async def get_entry(self, type: str, id: int) -> Union[Anime, Manga]:
+    async def search_character(
+        self, query: str, limit: int = 1
+    ) -> Optional[Union[Character, List[Character]]]:
         """|coro|
 
-        Get an entry object (`Anime` | `Manga`) by an id
+        Shortcut function to :meth:`search` with the `type` parameter populated by the "character" keyword
+
+        Parameters
+        -----------
+        query: :class:`str`
+            Represents the search query
+        limit: :class:`int`
+            Limit the search to a specific number of results
+        
+        Note
+        ------------
+        By searching characters, you will not get :attr:`askitsu.Character.media_id` and :attr:`askitsu.Character.role` attributes
+        """
+        return await self.search("characters", query=query, limit=limit)
+
+    async def get_entry(self, type: str, id: int) -> Union[Anime, Manga, Character]:
+        """|coro|
+
+        Get an entry object (`Anime` | `Manga` | `Character`) by an id
 
         Parameters
         -----------
@@ -142,14 +175,15 @@ class Client:
         id: :class:`int`
             ID of the media
         """
-        entry = self._entries.get(type)
-        fetched_data = await self._get_data(f"{BASE}/{type}/{id}")
-        return entry(type, fetched_data["data"])
+        type_lower = type.lower()
+        entry = self._entries.get(type_lower)
+        fetched_data = await self._get_data(f"{BASE}/{type_lower}/{id}")
+        return entry(attributes=fetched_data["data"], session=self._session) if fetched_data else None
 
     async def get_anime_entry(self, id: int) -> Anime:
         """|coro|
 
-        Shortcut for :coro:`get_entry`; returns only `Anime` object
+        Shortcut for :meth:`get_entry`; returns only `Anime` object
 
         Parameters
         -----------
@@ -158,6 +192,18 @@ class Client:
         """
         return await self.get_entry("anime", id=id)
 
+    async def get_manga_entry(self, id: int) -> Manga:
+        """|coro|
+
+        Shortcut for :meth:`get_entry`; returns only `Manga` object
+
+        Parameters
+        -----------
+        id: :class:`int`
+            ID of the manga
+        """
+        return await self.get_entry("manga", id=id)
+
     async def get_stream_links(self, anime: Anime) -> List[StreamLink]:
         """|coro|
 
@@ -169,14 +215,14 @@ class Client:
             The anime to get stream links
         """
         if not isinstance(anime, Anime):
-            return InvalidArgument
+            raise InvalidArgument(
+                f"{Fore.RED}'{anime}' is not an istance of Anime\n"
+                f"Make sure you pass a valid argument to {Fore.LIGHTCYAN_EX}get_stream_links{Style.RESET_ALL}"
+            )
         fetched_data = await self._get_data(
             url=f"{BASE}/anime/{anime.id}/streaming-links"
         )
-        stream_links = []
-        for links in fetched_data["data"]:
-            stream_links.append(StreamLink(links))
-        return stream_links
+        return [StreamLink(links) for links in fetched_data["data"]]
 
     async def get_characters(
         self, entry: Union[Anime, Manga], limit: int = 1
@@ -212,6 +258,56 @@ class Client:
         ]
         return characters if len(characters)>1 else characters[0]
 
+    async def get_trending_entry(
+        self, type: Literal["anime", "manga"]
+    ) -> Union[List[Anime], List[Manga]]:
+        """|coro|
+
+        Return a list of anime or manga
+
+        Parameters
+        -----------
+        entry: Union[:class:`Anime`, :class:`Manga`]
+            Entry to fetch its trending
+        """
+        type_lower = type.lower()
+        if type_lower == "characters":
+            raise InvalidArgument(
+                f"{Fore.RED}Characters can't be fetched in trending list\n"
+                f"Please pass 'anime' or 'manga' as parameter to {Fore.LIGHTCYAN_EX}get_trending_entry{Style.RESET_ALL}"
+            )
+        entry = self._entries.get(type_lower)
+        fetched_data = await self._get_data(
+            f"{BASE}/trending/{type_lower}"
+        )
+        return [entry(attributes=attributes, session=self._session) 
+            for attributes in fetched_data["data"]
+        ]
+        
+    async def get_reviews(
+        self, entry: Union[Manga, Anime],
+        limit: int = 1
+    ) -> Optional[Union[Review, List[Review]]]:
+        """|coro|
+
+        Get reviews of a given entry
+
+        Parameters
+        -----------
+        entry: Union[Manga, Anime]
+            A valid entry to get reviews
+        limit: :class:`int`
+            Limit to reviews to fetch
+        """
+        fetched_data = await self._get_data(
+            url=f"{BASE}/{entry.entry_type}/{entry.id}/reviews?page%5Blimit%5D={limit}"
+        )
+        reviews = [
+            Review(entry.id, entry.entry_type, reviews) 
+            for reviews in fetched_data["data"]
+        ]
+        return (reviews if limit>1 else reviews[0]) if reviews else None
+
     async def close(self) -> None:
         """
         Close client connection
